"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var mustreams_1 = require("mustreams");
var helper_1 = require("../test/helper");
function createWriteStreams(numStreams) {
    var result = new Array(numStreams);
    for (var i = 0; i < numStreams; ++i) {
        result[i] = new mustreams_1.MuWriteStream(1);
    }
    return result;
}
exports.createWriteStreams = createWriteStreams;
function createReadStreams(outStreams) {
    var result = new Array(outStreams.length);
    for (var i = 0; i < outStreams.length; ++i) {
        result[i] = new mustreams_1.MuReadStream(outStreams[i].bytes());
        outStreams[i].destroy();
    }
    return result;
}
exports.createReadStreams = createReadStreams;
function genArray(muType, length) {
    var result = new Array(length);
    for (var i = 0; i < length; ++i) {
        result[i] = helper_1.randomValue(muType);
    }
    return result;
}
exports.genArray = genArray;
function genDictionary(muType, numProps) {
    function propName() {
        var length = 10;
        var chars = new Array(length);
        for (var i = 0; i < length; ++i) {
            chars[i] = Math.random() * 26 + 97 | 0;
        }
        return String.fromCharCode.apply(String, chars);
    }
    var result = {};
    for (var i = 0; i < numProps; ++i) {
        result[propName()] = helper_1.randomValue(muType);
    }
    return result;
}
exports.genDictionary = genDictionary;
function changeValues(dict, muType) {
    var result = JSON.parse(JSON.stringify(dict));
    Object.keys(result).forEach(function (k) {
        result[k] = helper_1.randomValue(muType);
    });
    return result;
}
exports.changeValues = changeValues;
function shallowMerge(target, source) {
    var result = JSON.parse(JSON.stringify(target));
    Object.keys(source).forEach(function (k) {
        result[k] = source[k];
    });
    return result;
}
exports.shallowMerge = shallowMerge;
function genStruct(spec) {
    var props = Object.keys(spec);
    var muTypes = props.map(function (p) { return spec[p]['muType']; });
    var result = {};
    props.forEach(function (p, idx) {
        result[p] = helper_1.randomValue(muTypes[idx]);
    });
    return result;
}
exports.genStruct = genStruct;
exports.muType2ArrayType = {
    float32: Float32Array,
    float64: Float64Array,
    int8: Int8Array,
    int16: Int16Array,
    int32: Int32Array,
    uint8: Uint8Array,
    uint16: Uint16Array,
    uint32: Uint32Array,
};
function genVector(muType, dimension) {
    var result = new exports.muType2ArrayType[muType](dimension);
    for (var i = 0; i < dimension; ++i) {
        result[i] = helper_1.randomValue(muType);
    }
    return result;
}
exports.genVector = genVector;
function calcContentBytes(outs) {
    function reducer(acc, out) {
        return acc + out.offset;
    }
    var sum = outs.reduce(reducer, 0);
    return sum / outs.length;
}
exports.calcContentBytes = calcContentBytes;
//# sourceMappingURL=gendata.js.map