"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var identityStr = ' ';
var MuFixedASCII = (function () {
    function MuFixedASCII(lengthOrIdentity) {
        this.muType = 'fixed-ascii';
        if (typeof lengthOrIdentity === 'number') {
            if (lengthOrIdentity < 0) {
                throw new RangeError('length cannot be negative');
            }
            var length_1 = lengthOrIdentity | 0;
            if (length_1 > 1 << 27) {
                throw new RangeError('invalid length');
            }
            this.length = length_1;
            while (length_1 > identityStr.length) {
                identityStr += identityStr;
            }
            this.identity = identityStr.substr(0, length_1);
        }
        else {
            this.identity = lengthOrIdentity;
            this.length = lengthOrIdentity.length;
        }
        this.json = {
            type: 'fixed-ascii',
            identity: this.identity,
        };
    }
    MuFixedASCII.prototype.alloc = function () { return this.identity; };
    MuFixedASCII.prototype.free = function (_) { };
    MuFixedASCII.prototype.equal = function (x, y) {
        return x === y;
    };
    MuFixedASCII.prototype.clone = function (x) { return x; };
    MuFixedASCII.prototype.copy = function (source, target) { };
    MuFixedASCII.prototype.diff = function (base, target, out) {
        var length = this.length;
        if (base.length !== length) {
            throw new Error('base of invalid length');
        }
        if (target.length !== length) {
            throw new Error('target of invalid length');
        }
        if (base !== target) {
            out.grow(length);
            out.writeASCIINoLength(target);
            return true;
        }
        return false;
    };
    MuFixedASCII.prototype.patch = function (base, inp) {
        return inp.readASCIIOf(this.length);
    };
    return MuFixedASCII;
}());
exports.MuFixedASCII = MuFixedASCII;
//# sourceMappingURL=fixed-ascii.js.map