"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var test = require("tape");
var equal = require("fast-deep-equal");
var __1 = require("../");
var constants_1 = require("../constants");
var helper_1 = require("./helper");
test('dictionary - identity', function (t) {
    var dictA = new __1.MuDictionary(new __1.MuVoid());
    t.same(dictA.identity, {});
    var confession = {
        married: true,
        hadAffairs: true,
        havingAnAffair: true,
    };
    var dictB = new __1.MuDictionary(new __1.MuBoolean(), confession);
    t.equals(dictB.identity, confession);
    t.end();
});
function randomDictionary(depth, muType, genKey, numProps_) {
    var result = {};
    var numProps = numProps_ || Math.random() * 5 | 0;
    if (depth <= 1) {
        for (var i = 0; i < numProps; ++i) {
            result[genKey()] = helper_1.randomValue(muType);
        }
        return result;
    }
    for (var i = 0; i < numProps; ++i) {
        result[genKey()] = randomDictionary(depth - 1, muType, genKey, numProps_);
    }
    return result;
}
function genDictionary(depth, muType, numProps) {
    if (numProps === void 0) { numProps = 5; }
    var result = {};
    var charCode = 97;
    if (depth === 1) {
        for (var i = 0; i < numProps; ++i) {
            result[String.fromCharCode(charCode++)] = helper_1.randomValue(muType);
        }
        return result;
    }
    for (var i = 0; i < numProps; ++i) {
        result[String.fromCharCode(charCode++)] = genDictionary(depth - 1, muType, numProps);
    }
    return result;
}
test('dictionary - equal()', function (t) {
    for (var _i = 0, muPrimitiveTypes_1 = constants_1.muPrimitiveTypes; _i < muPrimitiveTypes_1.length; _i++) {
        var muType = muPrimitiveTypes_1[_i];
        var valueSchema = helper_1.muPrimitiveSchema(muType);
        if (valueSchema) {
            var dictionarySchema = new __1.MuDictionary(valueSchema);
            t.ok(dictionarySchema.equal({}, {}));
            var dictionary = randomDictionary(1, valueSchema, helper_1.randomString);
            t.ok(dictionarySchema.equal(dictionarySchema.clone(dictionary), dictionary));
            var a = genDictionary(1, muType);
            var b = genDictionary(1, muType);
            t.equal(dictionarySchema.equal(a, b), equal(a, b));
            a = randomDictionary(1, muType, helper_1.randomString, 5);
            b = randomDictionary(1, muType, helper_1.randomString, 5);
            t.equal(dictionarySchema.equal(a, b), equal(a, b));
            for (var i = 0; i < 10; ++i) {
                a = randomDictionary(1, muType, helper_1.randomString);
                b = randomDictionary(1, muType, helper_1.randomString);
                t.equal(dictionarySchema.equal(a, b), equal(a, b));
            }
        }
    }
    for (var _a = 0, muPrimitiveTypes_2 = constants_1.muPrimitiveTypes; _a < muPrimitiveTypes_2.length; _a++) {
        var muType = muPrimitiveTypes_2[_a];
        var valueSchema = helper_1.muPrimitiveSchema(muType);
        if (valueSchema) {
            var dictionarySchema = new __1.MuDictionary(new __1.MuDictionary(valueSchema));
            t.ok(dictionarySchema.equal({}, {}));
            var dictionary = randomDictionary(2, valueSchema, helper_1.randomString);
            t.ok(dictionarySchema.equal(dictionarySchema.clone(dictionary), dictionary));
            var a = genDictionary(2, muType);
            var b = genDictionary(2, muType);
            t.equal(dictionarySchema.equal(a, b), equal(a, b));
            a = randomDictionary(2, muType, helper_1.randomString, 5);
            b = randomDictionary(2, muType, helper_1.randomString, 5);
            t.equal(dictionarySchema.equal(a, b), equal(a, b));
            for (var i = 0; i < 10; ++i) {
                a = randomDictionary(2, muType, helper_1.randomString);
                b = randomDictionary(2, muType, helper_1.randomString);
                t.equal(dictionarySchema.equal(a, b), equal(a, b));
            }
        }
    }
    for (var _b = 0, muPrimitiveTypes_3 = constants_1.muPrimitiveTypes; _b < muPrimitiveTypes_3.length; _b++) {
        var muType = muPrimitiveTypes_3[_b];
        var valueSchema = helper_1.muPrimitiveSchema(muType);
        if (valueSchema) {
            var dictionarySchema = new __1.MuDictionary(new __1.MuDictionary(valueSchema));
            t.ok(dictionarySchema.equal({}, {}));
            var dictionary = randomDictionary(3, valueSchema, helper_1.randomString);
            t.ok(dictionarySchema.equal(dictionarySchema.clone(dictionary), dictionary));
            var a = genDictionary(3, muType);
            var b = genDictionary(3, muType);
            t.equal(dictionarySchema.equal(a, b), equal(a, b));
            a = randomDictionary(3, muType, helper_1.randomString, 5);
            b = randomDictionary(3, muType, helper_1.randomString, 5);
            t.equal(dictionarySchema.equal(a, b), equal(a, b));
            for (var i = 0; i < 10; ++i) {
                a = randomDictionary(3, muType, helper_1.randomString);
                b = randomDictionary(3, muType, helper_1.randomString);
                t.equal(dictionarySchema.equal(a, b), equal(a, b));
            }
        }
    }
    t.end();
});
test('dictionary (flat) - clone()', function (t) {
    for (var _i = 0, muPrimitiveTypes_4 = constants_1.muPrimitiveTypes; _i < muPrimitiveTypes_4.length; _i++) {
        var muType = muPrimitiveTypes_4[_i];
        var valueSchema = helper_1.muPrimitiveSchema(muType);
        if (valueSchema) {
            var dictSchema = new __1.MuDictionary(valueSchema);
            for (var i = 0; i < 200; ++i) {
                var dict = randomDictionary(1, muType, helper_1.randomString);
                var copy = dictSchema.clone(dict);
                t.notEquals(copy, dict);
                t.same(copy, dict);
            }
        }
    }
    t.end();
});
test('dictionary (nested) - clone()', function (t) {
    for (var _i = 0, muPrimitiveTypes_5 = constants_1.muPrimitiveTypes; _i < muPrimitiveTypes_5.length; _i++) {
        var muType = muPrimitiveTypes_5[_i];
        var valueSchema = helper_1.muPrimitiveSchema(muType);
        if (valueSchema) {
            var dictSchema = new __1.MuDictionary(new __1.MuDictionary(valueSchema));
            for (var i = 0; i < 100; ++i) {
                var dict = randomDictionary(2, muType, helper_1.randomString);
                var copy = dictSchema.clone(dict);
                t.notEquals(copy, dict);
                t.same(copy, dict);
            }
            dictSchema = new __1.MuDictionary(new __1.MuDictionary(new __1.MuDictionary(valueSchema)));
            for (var i = 0; i < 100; ++i) {
                var dict = randomDictionary(3, muType, helper_1.randomString);
                var copy = dictSchema.clone(dict);
                t.notEquals(copy, dict);
                t.same(copy, dict);
            }
        }
    }
    t.end();
});
test('dictionary - copy()', function (t) {
    for (var _i = 0, muPrimitiveTypes_6 = constants_1.muPrimitiveTypes; _i < muPrimitiveTypes_6.length; _i++) {
        var muType = muPrimitiveTypes_6[_i];
        var valueSchema = helper_1.muPrimitiveSchema(muType);
        if (valueSchema) {
            var source = void 0;
            var target = void 0;
            var dictionarySchema = new __1.MuDictionary(valueSchema);
            for (var i = 0; i < 10; ++i) {
                source = randomDictionary(1, muType, helper_1.randomString);
                target = randomDictionary(1, muType, helper_1.randomString);
                dictionarySchema.copy(source, target);
                t.deepEqual(target, source);
            }
            dictionarySchema = new __1.MuDictionary(new __1.MuDictionary(valueSchema));
            for (var i = 0; i < 10; ++i) {
                source = randomDictionary(2, muType, helper_1.randomString);
                target = randomDictionary(2, muType, helper_1.randomString);
                dictionarySchema.copy(source, target);
                t.deepEqual(target, source);
            }
            dictionarySchema = new __1.MuDictionary(new __1.MuDictionary(new __1.MuDictionary(valueSchema)));
            for (var i = 0; i < 10; ++i) {
                source = randomDictionary(3, muType, helper_1.randomString);
                target = randomDictionary(3, muType, helper_1.randomString);
                dictionarySchema.copy(source, target);
                t.deepEqual(target, source);
            }
        }
    }
    t.end();
});
test('dictionary (flat) - diff() & patch()', function (t) {
    for (var _i = 0, muPrimitiveTypes_7 = constants_1.muPrimitiveTypes; _i < muPrimitiveTypes_7.length; _i++) {
        var muType = muPrimitiveTypes_7[_i];
        var valueSchema = helper_1.muPrimitiveSchema(muType);
        if (valueSchema) {
            var dictSchema = new __1.MuDictionary(valueSchema);
            var testPatchingPair = helper_1.testPatchingPairFactory(t, dictSchema);
            for (var i = 0; i < 200; ++i) {
                testPatchingPair(randomDictionary(1, muType, helper_1.randomString), randomDictionary(1, muType, helper_1.randomString));
            }
            for (var i = 0; i < 200; ++i) {
                testPatchingPair(randomDictionary(1, muType, helper_1.randomShortStr), randomDictionary(1, muType, helper_1.randomShortStr));
            }
            var testPatching = helper_1.testPatchingFactory(t, dictSchema);
            var dict = randomDictionary(1, muType, helper_1.randomString);
            testPatching(dict, dict);
        }
    }
    t.end();
});
test('dictionary (nested) - diff() & patch()', function (t) {
    for (var _i = 0, muPrimitiveTypes_8 = constants_1.muPrimitiveTypes; _i < muPrimitiveTypes_8.length; _i++) {
        var muType = muPrimitiveTypes_8[_i];
        var valueSchema = helper_1.muPrimitiveSchema(muType);
        if (valueSchema) {
            var dictSchema = new __1.MuDictionary(new __1.MuDictionary(valueSchema));
            var testPatchingPair = helper_1.testPatchingPairFactory(t, dictSchema);
            for (var i = 0; i < 200; ++i) {
                testPatchingPair(randomDictionary(2, muType, helper_1.randomShortStr), randomDictionary(2, muType, helper_1.randomShortStr));
            }
            dictSchema = new __1.MuDictionary(new __1.MuDictionary(new __1.MuDictionary(valueSchema)));
            testPatchingPair = helper_1.testPatchingPairFactory(t, dictSchema);
            for (var i = 0; i < 200; ++i) {
                testPatchingPair(randomDictionary(3, muType, helper_1.randomShortStr), randomDictionary(3, muType, helper_1.randomShortStr));
            }
            dictSchema = new __1.MuDictionary(new __1.MuDictionary(new __1.MuDictionary(new __1.MuDictionary(valueSchema))));
            testPatchingPair = helper_1.testPatchingPairFactory(t, dictSchema);
            for (var i = 0; i < 200; ++i) {
                testPatchingPair(randomDictionary(4, muType, helper_1.randomShortStr), randomDictionary(4, muType, helper_1.randomShortStr));
            }
            var testPatching = helper_1.testPatchingFactory(t, dictSchema);
            var dict = randomDictionary(4, muType, helper_1.randomShortStr);
            testPatching(dict, dict);
        }
    }
    t.end();
});
//# sourceMappingURL=dictionary.js.map