"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var test = require("tape");
var __1 = require("../");
var mustreams_1 = require("mustreams");
var constants_1 = require("../constants");
var helper_1 = require("./helper");
test('vector - identity', function (t) {
    var vecSchema = new __1.MuVector(new __1.MuFloat64(5e-324), 3);
    t.same(vecSchema.identity, [5e-324, 5e-324, 5e-324]);
    t.end();
});
test('vector - alloc() when the pool is empty', function (t) {
    var vecSchema = new __1.MuVector(new __1.MuUint8(), 5);
    var uint8 = vecSchema.alloc();
    t.equals(uint8.constructor, Uint8Array);
    t.equals(uint8.length, 5);
    t.end();
});
function randomTypedArray(muType, length) {
    var result = new constants_1.muType2TypedArray[muType](length);
    for (var i = 0; i < length; ++i) {
        result[i] = helper_1.randomValue(muType);
    }
    return result;
}
var muNumberTypes = [
    'float32',
    'float64',
    'int8',
    'int16',
    'int32',
    'uint8',
    'uint16',
    'uint32',
];
test('vector - equal()', function (t) {
    var valueSchema = helper_1.muNumberSchema('uint8');
    var vectorSchema = new __1.MuVector(valueSchema, 1);
    var a = new Uint8Array([0]);
    var b = new Uint8Array([0]);
    t.ok(vectorSchema.equal(a, b));
    b[0] = 1;
    t.notOk(vectorSchema.equal(a, b));
    var c = new Uint8Array([0, 0]);
    t.notOk(vectorSchema.equal(a, c));
    var d = new Int8Array([0]);
    t.notOk(vectorSchema.equal(a, d));
    var f = new Int8Array([0]);
    t.notOk(vectorSchema.equal(d, f));
    t.end();
});
test('vector - clone()', function (t) {
    for (var _i = 0, muNumberTypes_1 = muNumberTypes; _i < muNumberTypes_1.length; _i++) {
        var muType = muNumberTypes_1[_i];
        var valueSchema = helper_1.muNumberSchema(muType);
        var dimension = 100;
        var vecSchema = new __1.MuVector(valueSchema, dimension);
        for (var i = 0; i < 200; ++i) {
            var vec = randomTypedArray(muType, dimension);
            var copy = vecSchema.clone(vec);
            t.notEquals(copy, vec);
            t.same(copy, vec);
        }
    }
    t.end();
});
test('vector - diff() & patch()', function (t) {
    for (var _i = 0, muNumberTypes_2 = muNumberTypes; _i < muNumberTypes_2.length; _i++) {
        var muType = muNumberTypes_2[_i];
        var valueSchema = helper_1.muNumberSchema(muType);
        var dimension = 5;
        var vecSchema = new __1.MuVector(valueSchema, dimension);
        var testPatchingPair = helper_1.testPatchingPairFactory(t, vecSchema);
        for (var i = 0; i < 200; ++i) {
            testPatchingPair(randomTypedArray(muType, dimension), randomTypedArray(muType, dimension));
        }
    }
    t.end();
});
test('random test', function (t) {
    var testSchema = new __1.MuVector(new __1.MuFloat32(0.5), 3);
    function randomVector() {
        var result = testSchema.alloc();
        result[0] = ((Math.random() * 4) / 2) - 1;
        result[1] = ((Math.random() * 4) / 2) - 1;
        result[2] = ((Math.random() * 4) / 2) - 1;
        return result;
    }
    function calcDiff(a, b) {
        var x = new mustreams_1.MuWriteStream(1);
        testSchema.diff(a, b, x);
        return new mustreams_1.MuReadStream(x.bytes());
    }
    function testPair(a, b) {
        var x = calcDiff(a, b);
        if (x.length === 0) {
            t.same(a, b, 'vectors are equal');
        }
        else {
            var c = testSchema.patch(a, x);
            t.same(b, c, 'patch ok');
            t.equals(x.length, x.offset, 'offset ok');
        }
    }
    for (var i = 0; i < 100; ++i) {
        var as = randomVector();
        var bs = randomVector();
        testPair(as, bs);
        testPair(bs, as);
        testPair(testSchema.identity, as);
        testPair(testSchema.identity, bs);
    }
    t.end();
});
//# sourceMappingURL=vector.js.map