"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var MuSchemaLogger = (function () {
    function MuSchemaLogger(schema, name) {
        this.allocCount = 0;
        this.freeCount = 0;
        this._savedAlloc = 0;
        this._savedFree = 0;
        this._schema = schema;
        this.name = name;
    }
    MuSchemaLogger.prototype.begin = function () {
        this._savedAlloc = this._schema.allocCount;
        this._savedFree = this._schema.freeCount;
    };
    MuSchemaLogger.prototype.end = function () {
        this.allocCount += this._schema.allocCount - this._savedAlloc;
        this.freeCount += this._schema.freeCount - this._savedFree;
    };
    return MuSchemaLogger;
}());
exports.MuSchemaLogger = MuSchemaLogger;
var MuSchemaTrace = (function () {
    function MuSchemaTrace(base) {
        this.logs = [];
        this.allocCount = 0;
        this.freeCount = 0;
        this.schema = base;
        this.identity = base.identity;
        this.muType = base.muType;
        this.muData = base.muData;
        this.json = base.json;
    }
    MuSchemaTrace.prototype.createLog = function (name) {
        var x = new MuSchemaLogger(this, name);
        this.logs.push(x);
        return x;
    };
    MuSchemaTrace.prototype.printLog = function () {
        var x = this.logs.map(function (_a) {
            var name = _a.name, allocCount = _a.allocCount, freeCount = _a.freeCount;
            return {
                name: name,
                alloc: allocCount,
                free: freeCount,
                object: allocCount - freeCount,
            };
        });
        x.push({
            name: 'total',
            alloc: this.allocCount,
            free: this.freeCount,
            object: this.allocCount - this.freeCount,
        });
        if (console.table) {
            console.table(x);
        }
        else {
            console.log(x.map(function (y) { return JSON.stringify(y); }).join('\n'));
        }
    };
    MuSchemaTrace.prototype.alloc = function () {
        this.allocCount++;
        return this.schema.alloc();
    };
    MuSchemaTrace.prototype.free = function (x) {
        this.freeCount++;
        return this.schema.free(x);
    };
    MuSchemaTrace.prototype.equal = function (x, y) {
        return this.schema.equal(x, y);
    };
    MuSchemaTrace.prototype.clone = function (x) {
        this.allocCount++;
        return this.schema.clone(x);
    };
    MuSchemaTrace.prototype.copy = function (source, target) {
        this.schema.copy(source, target);
    };
    MuSchemaTrace.prototype.diff = function (base, target, out) {
        return this.schema.diff(base, target, out);
    };
    MuSchemaTrace.prototype.patch = function (base, inp) {
        this.allocCount++;
        return this.schema.patch(base, inp);
    };
    return MuSchemaTrace;
}());
exports.MuSchemaTrace = MuSchemaTrace;
//# sourceMappingURL=trace.js.map