"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
function isUnion(x) {
    if (x !== Object(x)) {
        return false;
    }
    if (Object.keys(x).length !== 2) {
        return false;
    }
    var hasOwnProperty = Object.prototype.hasOwnProperty;
    if (!hasOwnProperty.call(x, 'type')) {
        return false;
    }
    if (!hasOwnProperty.call(x, 'data')) {
        return false;
    }
    return true;
}
var MuUnion = (function () {
    function MuUnion(schemaSpec, identityType) {
        var _this = this;
        this.muType = 'union';
        this._types = Object.keys(schemaSpec);
        this.muData = schemaSpec;
        if (identityType) {
            this.identity = {
                type: identityType,
                data: schemaSpec[identityType].identity,
            };
        }
        else {
            this.identity = {
                type: '',
                data: void 0,
            };
        }
        var result = {};
        Object.keys(schemaSpec).forEach(function (subtype) {
            result[subtype] = _this.muData[subtype].json;
        });
        this.json = {
            type: 'union',
            identity: this.identity.type,
            data: result,
        };
    }
    MuUnion.prototype.alloc = function () {
        return {
            type: '',
            data: void 0,
        };
    };
    MuUnion.prototype.free = function (data) {
        this.muData[data.type].free(data.data);
    };
    MuUnion.prototype.equal = function (x, y) {
        if (!isUnion(x) || !isUnion(y)) {
            return false;
        }
        if (x.type !== y.type) {
            return false;
        }
        if (x.type === '') {
            return true;
        }
        return this.muData[x.type].equal(x.data, y.data);
    };
    MuUnion.prototype.clone = function (data) {
        var schema = this.muData[data.type];
        return {
            type: data.type,
            data: schema.clone(data.data),
        };
    };
    MuUnion.prototype.copy = function (source, target) {
        if (source === target) {
            return;
        }
        if (source.type === target.type) {
            this.muData[source.type].copy(source.data, target.data);
        }
        target.type = source.type;
        this.muData[target.type].free(target.data);
        target.data = this.muData[source.type].clone(source.data);
    };
    MuUnion.prototype.diff = function (base, target, stream) {
        stream.grow(8);
        var trackerOffset = stream.offset;
        ++stream.offset;
        var tracker = 0;
        var dataSchema = this.muData[target.type];
        if (base.type === target.type) {
            if (dataSchema.diff(base.data, target.data, stream)) {
                tracker = 1;
            }
        }
        else {
            stream.writeUint8(this._types.indexOf(target.type));
            if (dataSchema.diff(dataSchema.identity, target.data, stream)) {
                tracker = 2;
            }
            else {
                tracker = 4;
            }
        }
        if (tracker) {
            stream.writeUint8At(trackerOffset, tracker);
            return true;
        }
        stream.offset = trackerOffset;
        return false;
    };
    MuUnion.prototype.patch = function (base, stream) {
        var result = this.clone(base);
        var tracker = stream.readUint8();
        if (tracker & 1) {
            result.data = this.muData[result.type].patch(result.data, stream);
        }
        else if (tracker & 2) {
            result.type = this._types[stream.readUint8()];
            var schema = this.muData[result.type];
            result.data = schema.patch(result.data, stream);
        }
        else {
            result.type = this._types[stream.readUint8()];
            var schema = this.muData[result.type];
            result.data = schema.clone(schema.identity);
        }
        return result;
    };
    return MuUnion;
}());
exports.MuUnion = MuUnion;
//# sourceMappingURL=union.js.map