"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var constants_1 = require("./constants");
var MuVector = (function () {
    function MuVector(valueSchema, dimension) {
        this._pool = [];
        this.muType = 'vector';
        this._constructor = constants_1.muType2TypedArray[valueSchema.muType];
        this.identity = new this._constructor(dimension);
        for (var i = 0; i < dimension; ++i) {
            this.identity[i] = valueSchema.identity;
        }
        this.muData = valueSchema;
        this.dimension = dimension;
        this.json = {
            type: 'vector',
            valueType: this.muData.json,
            dimension: dimension,
        };
    }
    MuVector.prototype.alloc = function () {
        return this._pool.pop() || new this._constructor(this.dimension);
    };
    MuVector.prototype.free = function (vec) {
        this._pool.push(vec);
    };
    MuVector.prototype.equal = function (x, y) {
        if (!(x instanceof this._constructor) || !(y instanceof this._constructor)) {
            return false;
        }
        if (x.length !== y.length) {
            return false;
        }
        for (var i = x.length - 1; i >= 0; --i) {
            if (x[i] !== y[i]) {
                return false;
            }
        }
        return true;
    };
    MuVector.prototype.clone = function (vec) {
        var copy = this.alloc();
        copy.set(vec);
        return copy;
    };
    MuVector.prototype.copy = function (source, target) {
        if (source === target) {
            return;
        }
        target.set(source);
    };
    MuVector.prototype.diff = function (base_, target_, stream) {
        var base = new Uint8Array(base_.buffer);
        var target = new Uint8Array(target_.buffer);
        var dimension = this.identity.byteLength;
        stream.grow(Math.ceil(this.identity.byteLength * 9 / 8));
        var headPtr = stream.offset;
        var trackerOffset = headPtr;
        stream.offset = trackerOffset + Math.ceil(dimension / 8);
        var tracker = 0;
        var numPatch = 0;
        for (var i = 0; i < dimension; ++i) {
            if (base[i] !== target[i]) {
                stream.writeUint8(target[i]);
                tracker |= 1 << (i & 7);
                ++numPatch;
            }
            if ((i & 7) === 7) {
                stream.writeUint8At(trackerOffset++, tracker);
                tracker = 0;
            }
        }
        if (numPatch === 0) {
            stream.offset = headPtr;
            return false;
        }
        if (dimension & 7) {
            stream.writeUint8At(trackerOffset, tracker);
        }
        return true;
    };
    MuVector.prototype.patch = function (base, stream) {
        var resultArray = this.clone(base);
        var result = new Uint8Array(resultArray.buffer);
        var trackerOffset = stream.offset;
        var trackerBits = this.dimension * this.identity.BYTES_PER_ELEMENT;
        var trackerFullBytes = Math.floor(trackerBits / 8);
        var trackerBytes = Math.ceil(trackerBits / 8);
        stream.offset = trackerOffset + trackerBytes;
        for (var i = 0; i < trackerFullBytes; ++i) {
            var start = i * 8;
            var tracker = stream.readUint8At(trackerOffset + i);
            for (var j = 0; j < 8; ++j) {
                if (tracker & (1 << j)) {
                    result[start + j] = stream.readUint8();
                }
            }
        }
        if (trackerBits & 7) {
            var start = trackerFullBytes * 8;
            var tracker = stream.readUint8At(trackerOffset + trackerFullBytes);
            var partialBits = trackerBits & 7;
            for (var j = 0; j < partialBits; ++j) {
                if (tracker & (1 << j)) {
                    result[start + j] = stream.readUint8();
                }
            }
        }
        return resultArray;
    };
    return MuVector;
}());
exports.MuVector = MuVector;
//# sourceMappingURL=vector.js.map